﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Newtonsoft.Json;
using SteamKit2;
using steam_dropler.Steam;

namespace steam_dropler.Model
{
    public class AccountConfig
    {
        /// <summary>
        /// Логин аккаунта
        /// </summary>
        [JsonIgnore]
        public string Name { get; set; }

        /// <summary>
        /// Имя бота
        /// </summary>
        public string Alias { get; set; }

        /// <summary>
        /// Пароль аккаунта
        /// </summary>
        public string Password { get; set; }

        /// <summary>
        /// Steam id
        /// </summary>
        public ulong? SteamId { get; set; }

        /// <summary>
        /// Участвует в фарме
        /// </summary>
        public bool IdleEnable { get; set; }

        /// <summary>
        /// Показывать статус "Online"
        /// </summary>
        public string ShowStatus { get; set; }

        /// <summary>
        /// Время последнего запуска
        /// </summary>
        public DateTime? LastRun { get; set; }

        /// <summary>
        /// Идлит ли сейчас бот
        /// </summary>
        public bool IdleNow { get; set; }

        /// <summary>
        /// Команда боту
        /// </summary>
        [JsonIgnore]
        public string Action { get; set; }

        /// <summary>
        /// Task Id
        /// </summary>
        [JsonIgnore]
        public string TaskID { get; set; }

        /// <summary>
        /// Мобильный аутентификатор
        /// </summary>
        [JsonIgnore]
        public MobileAuth MobileAuth { get; set; }

        /// <summary>
        /// Путь до файла настроек
        /// </summary>
        private string FilePath { get; }
        

        /// <summary>
        /// Время последнего обновления ключа для авторизации
        /// </summary>
        public uint Ltime { get; set; }

        /// <summary>
        /// Упрощенный вход sda
        /// </summary>
        public string SharedSecret { get; set; }

        /// <summary>
        /// Правила дропа
        /// </summary>
        //public List<(uint, ulong)> DropConfig { get; set; }
        public List<(uint, string)> DropConfig { get; set; }

        [JsonIgnore]
        public List<uint> AppIds => DropConfig?.Where(t => t.Item2[0] != '-').Select(t => t.Item1).ToList();
        //public List<uint> AppIds => DropConfig?.Select(t=>t.Item1).ToList();

        /// <summary>
        /// Время дропа
        /// </summary>
        public TimeConfig TimeConfig { get; set; }

        public string AccessToken { get; set; }
        
        /// <summary>
        /// Способ авторизации
        /// </summary>
        public AuthType AuthType { get; set; } = AuthType.WithSecretKey;

        /// <summary>
        /// Конструктор для json
        /// </summary>
        public AccountConfig()
        {
        }

        /// <summary>
        /// Конструктор по файлу
        /// </summary>
        /// <param name="path"></param>
        public AccountConfig(string path)
        {
           var obj = JsonConvert.DeserializeObject<AccountConfig>(File.ReadAllText(path));
            AuthType = obj.AuthType;
            Password = obj.Password;
            SteamId = obj.SteamId;
            IdleEnable = obj.IdleEnable;
            Action = "none";
            ShowStatus = obj.ShowStatus;
            if (ShowStatus == null)
            {
                ShowStatus = MainConfig.Config.ShowStatus;
            }
            //DropConfig = obj.DropConfig ?? new List<(uint, ulong)>();
            DropConfig = obj.DropConfig ?? new List<(uint, string)>();
            IdleNow = obj.IdleNow;
            LastRun = obj.LastRun ?? DateTime.MinValue;
            SharedSecret = obj.SharedSecret;
            if (SharedSecret != null)
            {
                MobileAuth = new MobileAuth {SharedSecret = obj.SharedSecret};
            }
            if (IdleNow) IdleNow = false;
            LastRun = DateTime.MinValue;
            Name = Path.GetFileNameWithoutExtension(path);
            Alias = obj.Alias ?? Name;
            AccessToken = obj.AccessToken;
            if(obj.Ltime != 0) {Ltime = obj.Ltime;} else Ltime = 1;
            Ltime = obj.Ltime;

            int naml = Alias.Length;
            if (naml > Util.unmax) {
                Util.unmax = naml;
            }
            TimeConfig = obj.TimeConfig ?? MainConfig.Config.TimeConfig ?? new TimeConfig {IdleTime = 1440, PauseBeatwinIdleTime = 1};
            if (MainConfig.Config.ChkIdleTimeOut > TimeConfig.IdleTime) TimeConfig.IdleTime = MainConfig.Config.ChkIdleTimeOut;
            FilePath = path;
        }

        /// <summary>
        /// Сохранить изменения
        /// </summary>
        public void Save()
        {
            File.WriteAllText(FilePath, JsonConvert.SerializeObject(this));
        }

    }
}
