﻿using System;
using System.IO;
using System.Runtime.CompilerServices;
using System.Collections.Generic;
using Newtonsoft.Json;

namespace steam_dropler.Model
{
    public class MainConfig
    {
        private static string ConfigPath = $"Configs{Path.DirectorySeparatorChar}MainConfig.json";
        public static string AsfCfgPath = $"Configs{Path.DirectorySeparatorChar}asf";

        [JsonIgnore]
        public static MainConfig Config { get; set; }

        public string MaFileFolder { get; set; }

        public string DropHistoryFolder { get; set; }

        public int ParallelCount { get; set; }

        public int StartTimeOut { get; set; }

        public int ChkIdleTimeOut { get; set; }

        public int CountForChange { get; set; }

        public string ShowStatus { get; set; }

        public string tColor { get; set; }

        public ulong OwnerID { get; set; }

        public TimeConfig TimeConfig { get; set; }

        [JsonIgnore]
        public bool SavCfg { get; set; }

        public static void Load()
        {
            var obj = JsonConvert.DeserializeObject<MainConfig>(File.ReadAllText(ConfigPath));
            Config = obj; Config.SavCfg = false;
            if (obj.StartTimeOut < 1) {Config.StartTimeOut = 1; Config.SavCfg = true;}
            if (obj.ChkIdleTimeOut != 0)
            {
                if (obj.ChkIdleTimeOut < 1)
                {
                    Config.ChkIdleTimeOut = 1;
                    Config.SavCfg = true;
                }
            }
            else
            {
                Config.ChkIdleTimeOut = 10;
                Config.SavCfg = true;
            }
            if (Config.ChkIdleTimeOut > Config.TimeConfig.IdleTime) {Config.TimeConfig.IdleTime = Config.ChkIdleTimeOut; Config.SavCfg = true;}
            if (obj.CountForChange == 0) {Config.CountForChange = 5; Config.SavCfg = true;}
            if (obj.ShowStatus == null) {Config.ShowStatus = "Online"; Config.SavCfg = true;}
            if (obj.tColor == null) {Config.tColor = "test"; Config.SavCfg = true;}
            if (obj.OwnerID == 0) {Config.OwnerID = 11111111111111111; Config.SavCfg = true;}
        }
        public static void Save()
        {
            string s = "{\n  \"maFileFolder\": \"" + Config.MaFileFolder;
            s += "\",\n  \"dropHistoryFolder\": \"" + Config.DropHistoryFolder;
            s += "\",\n  \"parallelCount\": " + Config.ParallelCount;
            s += ",\n  \"startTimeOut\": " + Config.StartTimeOut;
            s += ",\n  \"ChkIdleTimeOut\": " + Config.ChkIdleTimeOut;
            s += ",\n  \"CountForChange\": " + Config.CountForChange;
            s += ",\n  \"ShowStatus\": \"" + Config.ShowStatus;
            s += "\",\n  \"tColor\": \"" + Config.tColor;
            s += "\",\n  \"OwnerID\": " + Config.OwnerID;
            s += ",\n  \"timeConfig\": {\n    \"IdleTime\": " + Config.TimeConfig.IdleTime;
            s += ",\n    \"PauseBeatwinIdleTime\": " + Config.TimeConfig.PauseBeatwinIdleTime;
            s += "\n  }\n}\n";
            File.WriteAllText(ConfigPath, s);
        }
    }
}
