﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.Threading;
using System.Threading.Tasks;
using Newtonsoft.Json;
using steam_dropler.Model;
using SteamKit2;
using SteamKit2.Internal;


namespace steam_dropler.Steam
{
    public class SteamMachine
    {

        public bool IsWork => _work;

        public SteamID SteamID => _client.SteamID;
        public uint AccountId => _client.SteamID.AccountID;

        public SteamConfiguration SteamConfiguration { get; private set; }
        public bool IsConnectedAndLoggedOn => _client?.SteamID != null;

        private readonly AccountConfig _steamAccount;
        private readonly SteamLoginHandler _loginHandler;
        private readonly SteamClient _client;
        private readonly SteamFriends _friends;
        private readonly SteamApps _steamApps;

        private readonly SteamUnifiedMessages.UnifiedService<IInventory> _inventoryService;
        private readonly SteamUnifiedMessages.UnifiedService<IDeviceAuth> _deviceService;
        private readonly SteamUnifiedMessages.UnifiedService<IPlayer> UnifiedPlayerService;

        private bool _work = true;

        public SteamMachine(AccountConfig steamAccount)
        {
            _client = new SteamClient();
            SteamConfiguration = _client.Configuration;
            var manager = new CallbackManager(_client);


            var steamUnifiedMessages = _client.GetHandler<SteamUnifiedMessages>();
            _inventoryService = steamUnifiedMessages.CreateService<IInventory>();
            _deviceService = steamUnifiedMessages.CreateService<IDeviceAuth>();
            UnifiedPlayerService = steamUnifiedMessages.CreateService<IPlayer>();

            _steamAccount = steamAccount;
            _loginHandler = new SteamLoginHandler(_steamAccount, _client, manager);
            _steamApps = _client.GetHandler<SteamApps>();
            _friends = _client.GetHandler<SteamFriends>();

            Task.Run(() =>
            {
                while (_work)
                {
                    manager.RunWaitCallbacks(TimeSpan.FromSeconds(1));
                }

            });
        }

        public async Task<EResult> EasyIdling()
        {
            var servRec = SteamServerList.GetServerRecord();
            if (servRec == null) {
                Thread.Sleep(10000);
                _steamAccount.Action = "none";
                return EResult.NoConnection;
            }
            var res = await _loginHandler.Login(servRec);
            if (res == EResult.OK)
            {
                if (_steamAccount.Action != "Ber1") _steamAccount.LastRun = DateTime.UtcNow;
                int Itime = _steamAccount.TimeConfig.IdleTime - (int)(DateTime.UtcNow - _steamAccount.LastRun.Value).TotalMinutes;
                _steamAccount.IdleNow = true;
                _steamAccount.Action = "none";
                _steamAccount.Save();
                //var appId = _steamAccount.AppIds;
                ConsoleKey qkey = ConsoleKey.F24;
                await GetLevel().ConfigureAwait(false);
                //if (_steamAccount.AppIds.Any())
                //{
                    for (int i = 0; i < Itime / MainConfig.Config.ChkIdleTimeOut; i++) {
                        PlayGames(_steamAccount.AppIds);
                        await CheckTimeItemsList(_steamAccount.DropConfig);
                        for (int j = 0; j < 12 * MainConfig.Config.ChkIdleTimeOut; j++) {
                            if (_steamAccount.Action[0] == 'S') break;
                            if (_steamAccount.Action != "none") DoAction();
                            if (Console.KeyAvailable) qkey = Console.ReadKey(true).Key;
                            if (qkey == ConsoleKey.Q || _steamAccount.Action == "!exit") {
                                Console.WriteLine("Попытка завершить программу");
                                Process.GetCurrentProcess().Kill();
                            }
                            Thread.Sleep(5000);
                        }
                        if (_steamAccount.Action[0] == 'S') break;
                    }
                    if (_steamAccount.Action[0] != 'S') await CheckTimeItemsList(_steamAccount.DropConfig);
                    for (int j = 0; j < 12 * (Itime % MainConfig.Config.ChkIdleTimeOut); j++) {
                        if (_steamAccount.Action[0] == 'S') break;
                        if (_steamAccount.Action != "none") DoAction();
                        if (Console.KeyAvailable) qkey = Console.ReadKey(true).Key;
                        if (qkey == ConsoleKey.Q || _steamAccount.Action == "!exit") {
                            Console.WriteLine("Попытка завершить программу");
                            Process.GetCurrentProcess().Kill();
                        }
                        Thread.Sleep(5000);
                    }
                    StopGame();
                //}
                if (_steamAccount.Action != "Serr") _steamAccount.IdleNow = false;
                _steamAccount.Action = "none";
                _steamAccount.Save();
            }
            return res;
        }

        public async Task<T> Execute<T>(Func<SteamMachine, T> func)
        {
            var res = await _loginHandler.Login(SteamServerList.GetServerRecord());
            if (res == EResult.OK)
            {
                var ret = func(this);
                Console.WriteLine("Call LogOf");
                LogOf();
                return ret;
            }
            LogOf();
            return default(T);
        }

        public void LogOf()
        {
            //Console.WriteLine($"{Util.hdl(_steamAccount.Alias)}(Do LogOf) IsConnectedAndLoggedOn:{IsConnectedAndLoggedOn}");
            Thread.Sleep(5000);
            _work = false;
            _client.Disconnect();
            SteamServerList.ReleasServerRecord(_loginHandler.ServerRecord);

        }

        private async Task AddFreeLicense(List<uint> gamesIds)
        {
            var result = await _steamApps.RequestFreeLicense(gamesIds);
            Console.WriteLine($"{Util.hdl(_steamAccount.Alias)}GrantedApps: {string.Join(",", result.GrantedApps)}");
        }

        //private async Task CheckTimeItemsList(List<(uint, ulong)> pairs)
        private async Task CheckTimeItemsList(List<(uint, string)> pairs)
        {
            if (!IsConnectedAndLoggedOn) return;
            bool NoDrop = true; int p; string s; (uint, string) pair;
            for(int i = 0; i < pairs.Count; i++)
            {
                pair = pairs[i];
                if (pair.Item2[0] == '-' || pair.Item2[0] == '+') continue;
                p = pair.Item2.IndexOf(',');
                if (p > 0) {
                    s = pair.Item2.Substring(0, p);
                    pairs[i] = (pair.Item1, pair.Item2.Substring(p+1) + "," + s);
                } else s = pair.Item2;
                CInventory_ConsumePlaytime_Request reqkf = new CInventory_ConsumePlaytime_Request
                {
                    appid = pair.Item1,
                    itemdefid = Convert.ToUInt64(s)
                };

                try
                {
                    var responce = await _inventoryService.SendMessage(x => x.ConsumePlaytime(reqkf));
                    var result = responce.GetDeserializedResponse<CInventory_Response>();

                    if (result.item_json != "[]")
                    {
                        NoDrop = false;
                        try
                        {
                            var items = JsonConvert.DeserializeObject<DropResult[]>(result.item_json);
                            Console.WriteLine($"{Util.hdl(_steamAccount.Alias)}Game {pair.Item1} - Item received");
                            foreach (var item in items)
                            {
                                Util.LogDrop(_steamAccount.Name, pair.Item1, item, reqkf.itemdefid);
                            }
                        }
                        catch (Exception e)
                        {
                            Console.ForegroundColor = ConsoleColor.Red;
                            Console.WriteLine($"{Util.hdl(_steamAccount.Alias)}{e}");
                            Console.ResetColor();
                        }
                    }
                }
                catch (TaskCanceledException)
                {
                    Console.ForegroundColor = ConsoleColor.Red;
                    Console.WriteLine($"{Util.hdl(_steamAccount.Alias)}Task was canceled. Переподключение...");
                    Console.ResetColor();

                    _client.Disconnect();
                    await Task.Delay(5000);
                    _client.Connect();
                }
            }
            if (NoDrop && _steamAccount.AppIds.Count > 0) Console.WriteLine($"{Util.hdl(_steamAccount.Alias)}TryDrop");
        }

        private void StopGame()
        {
            //Console.WriteLine($"{Util.hdl(_steamAccount.Alias)}(StGm) IsConnectedAndLoggedOn:{IsConnectedAndLoggedOn}");
            if (!IsConnectedAndLoggedOn) return;
            var games = new ClientMsgProtobuf<CMsgClientGamesPlayed>(EMsg.ClientGamesPlayed);
            games.Body.games_played.Add(new CMsgClientGamesPlayed.GamePlayed()
            {
                game_id = 0
            });
            _client.Send(games);
        }

        private void PlayGames(List<uint> gamesIds)
        {
            var games = new ClientMsgProtobuf<CMsgClientGamesPlayed>(EMsg.ClientGamesPlayed);
            if (gamesIds.Count > 0) {
                foreach (var gameId in gamesIds)
                {
                    games.Body.games_played.Add(new CMsgClientGamesPlayed.GamePlayed
                    {
                        game_id = new GameID(gameId),
                    });
                }
            }
            else {
                games.Body.games_played.Add(new CMsgClientGamesPlayed.GamePlayed()
                {
                    game_id = 0
                });
            }
            _client.Send(games);
        }

        internal void DoAction() {
            if (_steamAccount.Action == "!play") {_steamAccount.Action = "none"; PlayGames(_steamAccount.AppIds);}
            else if (_steamAccount.Action == "!chst") {
                _steamAccount.Action = "none";
                if (_steamAccount.ShowStatus == "Online") {
                    _friends.SetPersonaState(EPersonaState.Online);
                    Console.Write($"{Util.hdl(_steamAccount.Alias)}");
                    Console.ForegroundColor = ConsoleColor.DarkGreen;
                    Console.WriteLine("Online");
                    Console.ResetColor();
                } else _friends.SetPersonaState(EPersonaState.Offline);
            }
        }

        internal async Task GetLevel() {
            if (!IsConnectedAndLoggedOn) return;
            CPlayer_GetGameBadgeLevels_Request request = new CPlayer_GetGameBadgeLevels_Request();
            SteamUnifiedMessages.ServiceMethodResponse response;
            try {
                response = await UnifiedPlayerService.SendMessage(x => x.GetGameBadgeLevels(request));
            } catch (Exception e) {
                Console.ForegroundColor = ConsoleColor.Red;
                Console.WriteLine($"{Util.hdl(_steamAccount.Alias)}{e}");
                Console.ResetColor();
                return;
            }
            if (response.Result != EResult.OK) {
                return;
            }
            //CPlayer_GetGameBadgeLevels_Response body = response.GetDeserializedResponse<CPlayer_GetGameBadgeLevels_Response>();
            //Console.WriteLine($"{Util.hdl(_steamAccount.Alias)}Level:{body.player_level}");
        }
    }
}
