﻿using System;
using System.Collections.Generic;
using System.Linq;
using steam_dropler.Model;
using SteamKit2;
using SteamKit2.Discovery;

namespace steam_dropler.Steam
{
    public static class SteamServerList
    {
        //private const int CountForChange = 5;

        private static Dictionary<ServerRecord, int> _servers;

        private static readonly List<ServerRecord> BadRecords = new List<ServerRecord>();

        static SteamServerList()
        {
            var serverList = new SmartCMServerList(SteamConfiguration.Create(b => b.WithProtocolTypes(ProtocolTypes.Tcp)));
            _servers = serverList.GetAllEndPoints().Where(t => t.ProtocolTypes.HasFlag(ProtocolTypes.Tcp)).ToDictionary(t => t, t => 0);
        }

        public static ServerRecord GetServerRecord()
        {
            //Console.Write("GetServerRecord: ");
            var validRecord = _servers
                .FirstOrDefault(t => t.Value < MainConfig.Config.CountForChange && !BadRecords.Contains(t.Key)).Key;
            if (validRecord == null)
            {
                var serverList = new SmartCMServerList(SteamConfiguration.Create(b => b.WithProtocolTypes(ProtocolTypes.Tcp)));
                _servers = serverList.GetAllEndPoints()
                    .Where(t => t.ProtocolTypes.HasFlag(ProtocolTypes.Tcp))
                    .ToDictionary(t => t, t => 0);
                BadRecords.Clear();
                Console.WriteLine($"Список серверов обновлен. Найдено серверов: {_servers.Count}");
                if (_servers.Count > 0) {return GetServerRecord();}
                else return null;
            }
            _servers[validRecord] += 1;

            //Console.WriteLine($"vRec:{validRecord.GetHost()}  Cnt:{_servers[validRecord]}");
            return validRecord;
        }

        public static void ReleasServerRecord(ServerRecord serverRecord)
        {
            if (serverRecord != null && _servers.ContainsKey(serverRecord))
            {
                var uses = _servers[serverRecord];
                uses -= 1;
                if (uses < 0)
                {
                    uses = 0;
                }
                _servers[serverRecord] = uses;
            }
        }


        public static void SetBadServer(ServerRecord record)
        {
            BadRecords.Add(record);
        }
    }
}
