﻿using System;
using System.IO;
using System.IO.Compression;
using System.Runtime.InteropServices;
using System.Threading;
using System.Text;
using Pastel;
using steam_dropler.Model;
using steam_dropler.Steam;

namespace steam_dropler
{
	public class Util
	{
		public static int unmax;
		public static long GetSystemUnixTime()
		{
			return (long)(DateTime.UtcNow.Subtract(new DateTime(1970, 1, 1))).TotalSeconds;
		}

		public static byte[] HexStringToByteArray(string hex)
		{
			int hexLen = hex.Length;
			byte[] ret = new byte[hexLen / 2];
			for (int i = 0; i < hexLen; i += 2)
			{
				ret[i / 2] = Convert.ToByte(hex.Substring(i, 2), 16);
			}
			return ret;
		}

		public static void LogDrop(string accountName, uint game, DropResult result, ulong itemDefId)
		{
		    if (!Directory.Exists(MainConfig.Config.DropHistoryFolder))
		    {
			Directory.CreateDirectory(MainConfig.Config.DropHistoryFolder);
		    }
		    if (File.Exists(Path.Combine(MainConfig.Config.DropHistoryFolder, accountName+".txt")))
		    {
			using (StreamWriter sw = File.AppendText(Path.Combine(MainConfig.Config.DropHistoryFolder, accountName+".txt")))
			{
			    sw.WriteLine($"{DateTime.Now}: {game} - Drop item: {result.ItemDefId} - TriggerItemDrop: {itemDefId}");
			}
		    }
		    else
		    {
			using (StreamWriter sw = File.CreateText(Path.Combine(MainConfig.Config.DropHistoryFolder, accountName+".txt")))
			{
			    sw.WriteLine($"{DateTime.Now}: {game} - Drop item: {result.ItemDefId} - TriggerItemDrop: {itemDefId}");
			}
		    }
		}

		public static string Unzip(byte[] bytes) {
		    using (var msi = new MemoryStream(bytes))
		    using (var mso = new MemoryStream()) {
		        using (var gs = new GZipStream(msi, CompressionMode.Decompress)) {
		            gs.CopyTo(mso);
		            //CopyTo(gs, mso);
		        }
		        return Encoding.UTF8.GetString(mso.ToArray());
		    }
		}

		public static string hdl(string uname, int mode = 1)
		{
		    if (mode == 1) {return DateTime.Now.ToLongTimeString() + " [" + uname + new string (' ', unmax - uname.Length) + "] ";}
		    else if (mode == 2) {return "<" + uname + new string (' ', unmax - uname.Length) + "> ";}
		    else if (mode == 3) {return "<" + uname + "> ";}
		    else if (mode == 4) {return "/code <" + uname + "> ";}
		    else if (mode == 5) {return "/code \u2063\ufeff<" + uname + "> ";}
		    else return "[" + uname + new string (' ', unmax - uname.Length) + "] ";
		}

		public static ConsoleColor gtConColor(byte r, byte g, byte b)
		{
		    ConsoleColor ret = 0;
		    double rr = r, gg = g, bb = b, delta = double.MaxValue;
		    foreach (ConsoleColor cc in Enum.GetValues(typeof(ConsoleColor)))
		    {
			var n = Enum.GetName(typeof(ConsoleColor), cc);
			var c = System.Drawing.Color.FromName(n == "DarkYellow" ? "Orange" : n); // bug fix
			var t = Math.Pow(c.R - rr, 2.0) + Math.Pow(c.G - gg, 2.0) + Math.Pow(c.B - bb, 2.0);
			if (t == 0.0)
			return cc;
			if (t < delta)
			{
			    delta = t;
			    ret = cc;
			}
		    }
		    return ret;
		}

		public static void ColTest(){
		    int r, g, b;
		    if (!chkOS()) {
			MainConfig.Config.tColor = "16";
			MainConfig.Config.SavCfg = true;
			return;
		    }
		    for (int gg = 0; gg < 256; gg = gg + 8) {
			g = gg; if (gg > 255) g = 255;
			for (int bb = 0; bb < 256; bb = bb + 8) {
			    if (gg < 127){
				r = gg; b = bb;
			    } else {
				r = 255 - g; b = bb;
			    }
			    Console.Write("█".Pastel("#" + r.ToString("X2") + g.ToString("X2") + b.ToString("X2")));
			}
			for (int rr = 0; rr < 256; rr = rr + 8) {
			    if (gg < 127){
				r = rr; b = 255 - rr;
			    } else {
				r = rr; b = 255 - rr;
			    }
			    Console.Write("█".Pastel("#" + r.ToString("X2") + g.ToString("X2") + b.ToString("X2")));
			}
			Console.WriteLine();
		    }
		    Console.WriteLine("Если вы видите квадрат с плавной цветовой гаммой,");
		    Console.WriteLine("значит ваш терминал поддерживает режим TrueColor.");
		    Console.WriteLine("Вы хотите использовать этот режим? y/n");
		    string rl = Console.ReadLine();
		    if (rl == "y") {MainConfig.Config.tColor = "On"; MainConfig.Config.SavCfg = true;}
		    else if (rl == "n") {MainConfig.Config.tColor = "16"; MainConfig.Config.SavCfg = true;}
		    Console.Clear();
		}

		public static bool chkOS(){
		    string osv = Environment.OSVersion.ToString();
//		    Console.WriteLine($"OSVersion: {osv}");
		    if (RuntimeInformation.IsOSPlatform(OSPlatform.Linux)) {return true;}
		    int n = osv.IndexOf("Windows NT 6.");
		    if (n > 0) {
			if (osv[n+13]>49) {return true;}
			else return false;
		    }
		    if (osv.IndexOf("Windows NT 10") > 0) return true;
		    return false;
		}

	}

	public static class APIEndpoints
	{
		public const string STEAMAPI_BASE = "https://api.steampowered.com";
		public const string COMMUNITY_BASE = "https://steamcommunity.com";
		public const string MOBILEAUTH_BASE = STEAMAPI_BASE + "/IMobileAuthService/%s/v0001";
		public static string MOBILEAUTH_GETWGTOKEN = MOBILEAUTH_BASE.Replace("%s", "GetWGToken");
		public const string TWO_FACTOR_BASE = STEAMAPI_BASE + "/ITwoFactorService/%s/v0001";
		public static string TWO_FACTOR_TIME_QUERY = TWO_FACTOR_BASE.Replace("%s", "QueryTime");
	}
}
