﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using System.Timers;
using System.Diagnostics;
using System.Reflection;
using Newtonsoft.Json;
using steam_dropler.Model;
using steam_dropler.Steam;
using SteamKit2;

namespace steam_dropler
{
    public static class Worker
    {
        private static string AccountPath = $"Configs{Path.DirectorySeparatorChar}Accounts";

        //private static HashSet<AccountConfig> _accounts;
        public static HashSet<AccountConfig> _accounts;

	private static Dictionary<string, MobileAuth> _mobileAuths;

        private static readonly Dictionary<string, Task> TaskDictionary = new Dictionary<string, Task>();

        private static Timer _timer;

        public static void Start()
        {
            _timer = new Timer(1000 * MainConfig.Config.StartTimeOut);
            _timer.Elapsed += CheckToAdd;
            _timer.Start();
            Console.ForegroundColor = ConsoleColor.Yellow;
            Console.WriteLine("               Steam-dropler (" + Assembly.GetExecutingAssembly().GetCustomAttribute<AssemblyInformationalVersionAttribute>().InformationalVersion.Substring(6) + ")");
            Console.WriteLine("--- Originally made by koperniki. Modified by VAL1aNT ---. Modified by koperniki");
            Console.ResetColor();
            Console.WriteLine($"Аккаунты запускаются с периодичностью {MainConfig.Config.StartTimeOut} секунд.");
            Console.WriteLine($"Количество аккаунтов на одном сервере: {MainConfig.Config.CountForChange}");
            Console.WriteLine($"Интервал проверки дропа: {MainConfig.Config.ChkIdleTimeOut} минут.");
            Console.WriteLine($"Всего аккаунтов {_accounts.Count}, будут фармиться {_accounts.Count(t=>t.IdleEnable && t.MobileAuth?.SharedSecret!=null)}");
        }

        public static void Run()
        {
            MainConfig.Load();
            LoadAccounts();
            LoadMaFiles();
            SetMaFiles();
            if (MainConfig.Config.tColor == "test") Util.ColTest();
            if (MainConfig.Config.SavCfg == true) MainConfig.Save();
            Start();
        }

        private static void CheckToAdd(object sender, ElapsedEventArgs e)
        {
            //Console.WriteLine($"AccCount:{_accounts.Count(t => t.IdleNow)}  TaskCount:{TaskDictionary.Count}");
            if (TaskDictionary.Count > MainConfig.Config.ParallelCount) return;
            if (_accounts.Count(t => t.Action[0] == 'B') > 0) return;
            if (_accounts.Count(t => t.IdleNow) > TaskDictionary.Count) {
                var stacs = _accounts.Where(t => t.IdleNow);
                foreach (var stac in stacs) {
                    try {var tid = TaskDictionary[stac.TaskID].Id;}
                    catch {
                        if ((stac.TimeConfig.IdleTime - (int)(DateTime.UtcNow - stac.LastRun.Value).TotalMinutes) > 0) {
                            stac.Action = "Ber1"; Console.WriteLine($"{Util.hdl(stac.Alias)}Restart");
                        } else stac.Action = "Busy";
                        stac.IdleNow = false; AddToIdlingQueue(stac);
                        return;
                    }
                }
            }

            var steamaccont = _accounts.FirstOrDefault(t =>
                t.LastRun < DateTime.UtcNow.AddMinutes(-(t.TimeConfig.IdleTime + t.TimeConfig.PauseBeatwinIdleTime)) & t.IdleEnable && !t.IdleNow);

            if (steamaccont != null)
            {
                steamaccont.Action = "Busy";
                AddToIdlingQueue(steamaccont);
            }
        }


        private static void AddToIdlingQueue(AccountConfig account)
        {
            var id = Guid.NewGuid().ToString();
            account.TaskID = id;

            TaskDictionary[id] = Task.Run(async () =>
            {
                try
                {
                    var machine = new SteamMachine(account);
                    var result = await machine.EasyIdling();
                    if (result != EResult.OK)
                    {
                        Console.ForegroundColor = ConsoleColor.Red;
                        Console.WriteLine($"{Util.hdl(account.Alias)}not login: {result}");
                        Console.ResetColor();
                    }
                    //Console.WriteLine($"{Util.hdl(account.Alias)}Machine LogOf");
                    machine.LogOf();
                    //TaskDictionary.Remove(id);
                }
                catch (Exception e)
                {
                    Console.ForegroundColor = ConsoleColor.Red;
                    Console.WriteLine($"{Util.hdl(account.Alias)}{e}");
                    Console.ResetColor();
                    throw;
                }
                TaskDictionary.Remove(id);
            });
        }

        private static void LoadAccounts()
        {
            _accounts = new HashSet<AccountConfig>(new AccontConfigComparer());

	        if (Directory.Exists(AccountPath))
	        {
		        var jsonPaths = Directory.GetFiles(AccountPath).Where(t => Path.GetExtension(t) == ".json");

		        foreach (var jsonPath in jsonPaths)
		        {
			    _accounts.Add(new AccountConfig(jsonPath));
		        }
                
	        }
	        else
	        {
		        throw new Exception("Account folder not exist");
	        }

        }

	    private static void LoadMaFiles()
	    {			
			var objects = new List<MobileAuth>();
		    if (!string.IsNullOrEmpty(MainConfig.Config.MaFileFolder) && Directory.Exists(MainConfig.Config.MaFileFolder))
		    {
				var maFilePaths = Directory.GetFiles(MainConfig.Config.MaFileFolder).Where(t => Path.GetExtension(t) == ".maFile");

			    foreach (var maFile in maFilePaths)
			    {
				    var obj = JsonConvert.DeserializeObject<MobileAuth>(File.ReadAllText(maFile));
					objects.Add(obj);
			    }
			    _mobileAuths = objects.ToDictionary(t => t.AccountName, t => t);

		    }
		    else
		    {
                _mobileAuths = new Dictionary<string, MobileAuth>();
                            Console.ForegroundColor = ConsoleColor.Red;
			    Console.WriteLine("MaFile folder not exist");
                            Console.ResetColor();
		    }

		}

	    private static void SetMaFiles()
	    {
		    foreach (var accountConfig in _accounts)
		    {
			    if (_mobileAuths.ContainsKey(accountConfig.Name))
			    {
				    accountConfig.MobileAuth = _mobileAuths[accountConfig.Name];
			    }
		    }
	    }
    }
}
